<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2013 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */
namespace Concrete\Package\GntMaxmind\Src;

defined('C5_EXECUTE') or die('Access Denied.');

use Package;

use GeoIp2\Database\Reader;

use GeoIP2\Exception\AddressNotFoundException;

use Log;
use \Illuminate\Filesystem\Filesystem;

class MaxmindDatabase
{
	public static function getDatabasePath()
	{
		return Package::getByHandle('gnt_maxmind')->getDatabasePath();
	}

	public static function canDownload()
	{
		// 5.6 had the url_fopen ability, but sadly 5.7 only usses curl_init (for now?) :/
        // return ini_get('allow_url_fopen') || function_exists('curl_init');
        return function_exists('curl_init');
	}

    public static function downloadAndInstallLatest()
    {
        if (!self::canDownload()) {
            return t('Can not download: curl not installed');
        }

        $url = "http://geolite.maxmind.com/download/geoip/database/GeoLite2-City.mmdb.gz";

        $fh = \Core::make('helper/file');

        # initially stolen from the file helper code, since there is no way to specify directly
        # to use curl instead of the default fopen method ...
        $datePrefix = strftime("%Y%m%d-%H%M%S", time());

		$path = self::getDatabasePath();
        $prefix = "$path/$datePrefix";

        $contents = $fh->getContents($url, 40);

        if (false === $contents) {
            return t("Unable to download maxmind city database from %s", "<a href=\"$url\" target=\"_blank\">$url</a>");
        }
        $gz = "${prefix}-maxmind-geoip2-city.mmdb.gz";

        $fh->clear($gz);
        $fh->append($gz, $contents);

        $r = self::install($gz, $type);

		$fs = new Filesystem;
		$fs->delete($gz);
        if ($r !== true) {
            return $r;
        }

        return true;
    }

    public static function install($src)
    {
        $fh = \Core::make('helper/file');
        $datePrefix = strftime("%Y%m%d-%H%M%S", time());

		$path = self::getDatabasePath();
        $prefix = "$path/$datePrefix";

        if (!file_exists($src)) {
            return t("file not found");
        }

        ob_start();
        $u = @readgzfile($src);

        if (false === $u) {
            ob_get_clean();

            return t("Invalid geolite2 city database content (%s not a valid gzip archive)", $src);
        }

        $file = "${prefix}-maxmind-geoip2-city.mmdb";
        $fh->clear($file);
        $fh->append($file, ob_get_clean());

        try {
            $r = self::_loadGeoIPFile($file);
            $record = $r->city('128.101.101.101');
            if (!$record) {
                throw new Exception(t('Invalid maxmind db file, can not find test IP'));
            }
            $record->country->name;
        } catch (Exception $e) {
			$fs = new FileSystem();
			$fs->delete($file);

            return t('Invalid geolite2 city database: %s', $e->getMessage());
        }
        // todo : test the database somehow ?
        return true;
    }

    protected static $_dbList = null;

    public static function refreshDBList()
    {
        self::$_dbList = null;
    }
    public static function listDatabases()
    {
        if (! self::$_dbList) {
            $fh = \Core::make('helper/file');

            $prefix = self::getDatabasePath();
            $dirlist = $fh->getDirectoryContents($prefix, array('README'));

            $countries = array();
            $cities    = array();

            foreach ($dirlist as $file) {
                # format : product-date.time.mmdb
                $match = '(\d{4}\d{2}\d{2}-\d{2}\d{2}\d{2})-maxmind-geoip2-city\.mmdb';

                if (!preg_match("/^$match\$/", $file)) {
                    continue;
                }

                $dateTime = preg_replace("/$match/", '\1', $file);
                $size = filesize("$prefix/$file");

                $cities[$dateTime] = array($file, t("%s MB", sprintf("%.2f", ($size/1024/1024))));
            }

            ksort($cities);
            self::$_dbList = $cities;
        }

        return self::$_dbList;
    }

    public static function deleteDatabase($db)
	{
		$path = self::getDatabasePath();
        $dbName = "$path/".basename($db)."-"."maxmind-geoip2-city.mmdb";
        if (!is_writable($dbName)) {
            return false;
        }
		$fs = new Filesystem();
		$fs->delete($dbName);

        $pkg = Package::getByHandle('gnt_maxmind');
        $pkg->fixCurrentKey();

        return $r;
    }

    public static function database()
    {
        $pkg = Package::getByHandle('gnt_maxmind');
        $key = $pkg->currentMaxMindKey();
        if (!$key) {
            return;
        }

        return self::loadGeoIPDB($key);
    }

    protected function _loadGeoIPFile($file)
    {
        return new Reader($file);
    }

    protected static $_dbCache = array();
    public static function loadGeoIPDB($key)
    {
        if (!isset(self::$_dbCache[$key])) {
			$path = self::getDatabasePath();
            self::$_dbCache[$key] = self::_loadGeoIPFile("$path/$key-maxmind-geoip2-city.mmdb");
        }

        return new MaxmindDatabase(self::$_dbCache[$key]);
    }

    protected $_reader = null;
    protected function __construct($reader)
    {
        $this->_reader = $reader;
    }

    protected $_recordCache = array();
    protected function _getRecord($ip)
    {
        if (!isset($this->_recordCache[$ip])) {
            $record = null;
            try {
                $record = $this->_reader->city($ip);
            } catch (AddressNotFoundException $e) {
            }
            $this->_recordCache[$ip] = $record;
        }

        return $this->_recordCache[$ip];
    }

	public function hasRecord($ip)
	{
		try {
			$record = $this->_getRecord($ip);
			if ($record) {
				return true;
			}
		} catch (AddressNotFoundException $e) {
			return false;
		}
		return false;
	}

    public function getCountries($ip)
    {
        $record = $this->_getRecord($ip);
        if (!$record || !$record->country || !$record->country->names) {
            return array();
        }

        return $record->country->names;
    }


    public function getCities($ip)
    {
        $record = $this->_getRecord($ip);
        if (!$record || !$record->city || !$record->city->names) {
            return array();
        }

        return $record->city->names;
    }

    public function getStandardCountryName($ip)
    {
        $record = $this->_getRecord($ip);
        if (!$record) {
            return;
        }

        $iso  = $record->country->isoCode;
        $name = $this->getCountryNameForLocale($ip);

        return "$name ($iso)";
    }


    public function getCountryName($ip)
    {
        $record = $this->_getRecord($ip);
        if (!$record) {
            return;
        }

        $record = $this->_reader->city($ip);

        return $record->country->name;
    }

    public function getStateName($ip)
    {
        $record = $this->_getRecord($ip);
        if (!$record) {
            return;
        }

        $record = $this->_reader->city($ip);

        return $record->mostSpecificSubdivision->name;
    }

    public function getStateNameForLocale($ip, $loc = null)
    {
        $record = $this->_getRecord($ip);
        if (!$record) {
            return;
        }

        $record = $this->_reader->city($ip);

        $names = $record->mostSpecificSubdivision->names;

        foreach ($this->_getLocalGuesses($loc) as $l) {
            if (isset($names[$l])) {
                return $names[$l];
            }
        }
        return $record->mostSpecificSubdivision->name;
    }

    public function getStateNames($ip)
    {
        $record = $this->_getRecord($ip);
        if (!$record) {
            return;
        }

        $record = $this->_reader->city($ip);

        return $record->mostSpecificSubdivision->names;
    }

    protected $_currentLocales = null;
    protected function _getCurrentLocales()
    {
        if (is_null($this->_currentLocales)) {
            $langs = preg_replace('/;.*/', '', $_SERVER['HTTP_ACCEPT_LANGUAGE']);
            $this->_currentLocales = explode(',', $langs);
        }

        return $this->_currentLocales;
    }

    protected function _getLocalGuesses($loc = null)
    {
        if (is_null($loc)) {
            $langs = $this->_getCurrentLocales();
        }

        if (count($loc) == 2) {
            $loc = strtolower($loc);
        }
        $loc = str_replace('_', '-', $loc);

        $locs = array($loc);
        $locs[] = "$loc-".strtoupper($loc);
        $locs[] = str_replace('-.*', '', $loc);

        return $locs;
    }

    public function getCountryNameForLocale($ip, $loc = null)
    {
        $record = $this->_getRecord($ip);
        if (!$record) {
            return;
        }

        $names = $record->country->names;

        foreach ($this->_getLocalGuesses($loc) as $l) {
            if (isset($names[$l])) {
                return $names[$l];
            }
        }

        return $record->country->name;
    }

    public function getCountryCode($ip)
    {
        $record = $this->_getRecord($ip);
        if (!$record) {
            return;
        }

        return $record->country->isoCode;
    }

    public function getStateCode($ip)
    {
        $record = $this->_getRecord($ip);
        if (!$record) {
            return;
        }

        return $record->mostSpecificSubdivision->isoCode;
    }


	public function getLongitude($ip)
	{
        $record = $this->_getRecord($ip);
        if (!$record || !$record->city) {
            return;
        }

        return $record->location->longitude;
	}

	public function getLatitude($ip)
	{
        $record = $this->_getRecord($ip);
        if (!$record || !$record->city) {
            return;
        }

        return $record->location->latitude;
	}


    public function getCityName($ip)
    {
        $record = $this->_getRecord($ip);
        if (!$record || !$record->city) {
            return;
        }

        return $record->city->name;
    }

    public function getCityNameForLocale($ip, $loc = null)
    {
        $record = $this->_getRecord($ip);
        if (!$record || !$record->city || !$record->city->name) {
            return;
        }

        $names = $record->city->names;

        foreach ($this->_getLocalGuesses($loc) as $l) {
            if (isset($names[$l])) {
                return $names[$l];
            }
        }

        return $record->city->name;
    }

    public static function customizeMessage($message, $ip = null)
    {
        $db = self::database();
        if (! $db) {
            return $message;
        }

		if (!$ip) {
			$pkg = Package::getByHandle('gnt_maxmind');
			$ip = $pkg->getCurrentIP();
		}


        $pattens = array();
        if ($db->getCountryCode($ip)) {
            $patterns = array(
                '/%COUNTRY_CODE%/' => $db->getCountryCode($ip),
                '/%COUNTRY%/'      => $db->getCountryNameForLocale($ip),
				'/%STATE%/'		   => $db->getStateNameForLocale($ip),
				'/%STATE_CODE%/'   => $db->getStateCode($ip),
                '/%CITY%/'         => $db->getCityNameForLocale($ip),
            );

            $matches = array();
            preg_match_all('/%COUNTRY_[^%]+%/', $message, $matches);
            foreach ($matches[0] as $match) {
                $l = preg_replace('/%COUNTRY_([^%]+)%/', '\1', $match);
                if ($l == 'CODE') {
                    continue;
                }
                $patterns["/%COUNTRY_$l%/"] = $db->getCountryNameForLocale($ip, $l);
            }

            $matches = array();
            preg_match_all('/%CITY_([^%])+%/', $message->prompt, $matches);
            foreach ($matches[0] as $match) {
                $l = preg_replace('/%CITY_([^%]+)%/', '\1', $match);
                if ($l == 'CODE') {
                    continue;
                }
                $patterns["/%CITY_$l%/"] = $db->getCityNameForLocale($ip, $l);
            }

            $matches = array();
            preg_match_all('/%STATE_([^%])+%/', $message->prompt, $matches);
            foreach ($matches[0] as $match) {
                $l = preg_replace('/%STATE_([^%]+)%/', '\1', $match);
                if ($l == 'CODE') {
                    continue;
                }
                $patterns["/%STATE_$l%/"] = $db->getStateNameForLocale($ip, $l);
            }
        }

		$ipPat = $ip;
		if (!$ipPat) {
			$ipPat = Package::getByHandle('gnt_maxmind')->getCurrentIP();
		}
        $patterns[',%IP%,'] = $ipPat;

        /*
        XXX Currently Magic Data (from John) is not available for 5.7,
            this part will need an update when MD makes it to the market;
        Magic Data Integration
        Evaluates MD tokens in the message
        (The helper checks internally for MD installation)

        */
		// Code to review when MD really gets done?
        // $mdsh = Core::make(('helper/magic_data_symbols', 'gnt_maxmind');
        // message = $mdsh->fill($message);

        return preg_replace(array_keys($patterns), array_values($patterns), $message);
    }
}

// vim: set noexpandtab ts=4 :

