<?php 
/**
 *-----------------------------------------------------------------------------*
 * License: Concrete5 marketplace :                                            *
 * http://www.concrete5.org/help/legal/commercial_add-on_license/              *
 *                                                                             *
 * @author (Florian Delizy)                                                    *
 *                                                                             *
 * Copyright(C) 2015 Florian Delizy <florian.delizy@gmail.com>                 *
 *-----------------------------------------------------------------------------*
 */
namespace Concrete\Package\GntMaxmind\Src;

defined('C5_EXECUTE') or die('Access Denied.');

use Database;
use Package;
use Page;
use Log;

class RedirectRuleSet
{
    protected $_rules = null;
    protected $_routes = array();

    protected $_bID  = null;
    protected $_avID = null;

    protected function _getRule($ip = null)
    {
        if (!$ip) {
            $pkg = Package::getByHandle('gnt_maxmind');
            $ip = $pkg->getCurrentIP();
        }

        if (! array_key_exists($ip, $this->_routes)) {
            $db = MaxmindDatabase::database();
            if (!$db) {
                return;
            }

            $country = $db->getCountryCode($ip);
            $state   = $db->getStateCode($ip);
            $city    = $db->getCityNameForLocale($ip);

            $this->_routes[$ip] = null;
            foreach ($this->getRules() as $rule) {
                if (!$rule->route($ip, $country, $state, $city)) {
                    continue;
                }

                $this->_routes[$ip] = $rule;
                break;
            }
        }

        return $this->_routes[$ip];
    }

    public function haveRoute($ip = null)
    {
        return !is_null($this>_getRule($ip));
    }

    public function route($ip = null, $simulate = false)
    {
        $c = Page::getCurrentPage();
        if ($c && !$c->isError() && $c->isEditMode()) {
            return;
        } # disable for edit mode


        # global switch
        $pkg = Package::getByHandle('gnt_maxmind');
        if ($pkg->isDisabled()) {
            return;
        }

        $rule = $this->_getRule($ip);

        if (is_null($rule)) {
            return;
        }

        if (!$simulate) {
            $rule->performRedirection();
        }

        return $rule;
    }

    protected function _getRuleByID($rID)
    {
        if (!$rID) {
            return;
        }

        foreach ($this->getRules() as $rule) {
            if ($rule->ruleID() == $rID) {
                return $rule;
            }
        }

        $rule = RedirectRule::getByID($rID);
        if (!$rule || $rule->rsID() != $this->_rsID) {
            return;
        }

        $this->_rules[] = $rule;

        return $rule;
    }

    protected $_rsID        = null;
    protected $_message    = '';
    protected $_activated    = 1;
    protected $_name        = null;
    protected $_debug        = 0;

    protected $_fields = array( 'message', 'name' );
    protected $_checkboxes = array(  'activated', 'debug' );

    public function name()
    {
        return $this->_name;
    }
    public function activated()
    {
        return $this->_activated;
    }
    public function message()
    {
        return $this->_message;
    }
    public function getRuleSetID()
    {
        return $this->_rsID;
    }
    public function debug()
    {
        return $this->_debug;
    }

    public function setDebug($debug)
    {
        $this->_debug = $debug ? 1 : 0;
    }

    public function debugLog($rank, $message)
    {
        if (!$this->debug()) {
            return;
        }
        Log::addEntry('['.$this->_name.'#'.$rank.'] '.$message, 'gnt_maxmind');
    }

    protected function _getFields()
    {
        return array_merge($this->_fields, $this->_checkboxes);
    }

    public function setPropertiesFromArray(array $array)
    {
        $fields = $this->_getFields();
        if (is_null($this->_rsID)) {
            $fields[] = 'rsID';
        }
        foreach ($fields as $key) {
            $property = "_$key";
            if (in_array($key, $this->_checkboxes)) {
                $this->$property = 0;
            }
            if (isset($array[$key])) {
                $this->$property = $array["$key"];
            }
        }
    }

    public function getRecord()
    {
        $record = array();
        $fields = $this->_getFields();
        $fields[] = 'rsID';
        foreach ($fields as $key) {
            $property = "_$key";
            $record[$key] = $this->$property;
        }

        return $record;
    }

    public function save()
    {
        $record = $this->getRecord();

        $db = Database::getActiveConnection();

        if ($this->validate() !== true) {
            return false;
        }

        if (is_null($this->_rsID)) {
            $db->insert('gntMaxMindRedirectRuleSet', $record);
            $rs = $db->Execute('SELECT LAST_INSERT_ID()');
            $row = $rs->FetchRow();
            $this->_rsID = $row['LAST_INSERT_ID()'];
        } else {
            $db->update('gntMaxMindRedirectRuleSet', $record, array('rsID' => $this->_rsID));
        }

        foreach ($this->getRules() as $rule) {
            $rule->save();
        }
    }

    public function delete()
    {
        if (!$this->_rsID) {
            return;
        }
        foreach ($this->getRules() as $rule) {
            $rule->delete();
        }

        $q = "DELETE FROM gntMaxMindRedirectRuleSet WHERE rsID = $this->_rsID";
        Database::getActiveConnection()->Execute($q);
        $this->_rsID = null;
        $this->_rules = null;
    }

    public function readForm(array $args = null)
    {
        if (is_null($args) || !is_array($args) || !count($args)) {
            return;
        }
        $this->setPropertiesFromArray($args);

        $rules = $args['rules'];
        if (is_null($rules) || !is_array($rules) || !count($rules)) {
            return;
        }

        foreach ($rules as $formID => $desc) {
            $rID = $desc['rID'];
            if ($rID) {
                $rule = $this->_getRuleByID($desc['rID']);
            } else {
                $rule = new RedirectRule($this);
                $this->_rules[] = $rule;
                $rule->setFormID($formID);
            }
            if (!$rule) {
                continue;
            }
            $rule->setPropertiesFromArray($desc);
        }
    }

    protected $_error = null;
    protected function _saveError($err)
    {
        $this->_error = $err;
    }
    public function error()
    {
        return $this->_error;
    }

    public function validate()
    {
        if (!$this->_name) {
            $this->_saveError(t('Rulesets must have a name'));

            return false;
        }

        foreach ($this->getRules() as $rule) {
            if (!$rule->validate()) {
                return false;
            }
        }

        return true;
    }

    public static function getByID($rsID)
    {
        if (!$rsID) {
            return;
        }

        $q = "SELECT * FROM gntMaxMindRedirectRuleSet WHERE rsID = ?";
        $db = Database::getActiveConnection();
        $r = $db->Execute($q, array($rsID));
        if ($r->RowCount() == 0) {
            return;
        }
        $row = $r->FetchRow();

        $rs = new RedirectRuleSet();
        $rs->setPropertiesFromArray($row);

        return $rs;
    }

    public static function listRuleSets()
    {
        $q = "SELECT rsID, name FROM gntMaxMindRedirectRuleSet";
        $db = Database::getActiveConnection();
        $l = $db->GetAssoc($q);
        if (!count($l)) {
            return array();
        }

        $list = array( t('Select Rule Set') );
        foreach ($l as $k => $v) {
            $list[$k] = $v;
        }

        return $list;
    }

    public function refreshRules()
    {
        $this->_rules = null;
    }

    public function getRules()
    {
        if (is_null($this->_rules)) {
            $rules = array();
            if ($this->_rsID) {
                $q = "SELECT * FROM gntMaxMindRedirectRule WHERE rsID = '$this->_rsID' ORDER by rank";
                $db = Database::getActiveConnection();

                foreach ($db->GetAll($q) as $desc) {
                    $rule = new RedirectRule($this);
                    $rule->setPropertiesFromArray($desc);
                    $rules[] = $rule;
                }
            }
            $this->_rules = $rules;
        }

        return $this->_rules;
    }
}

